"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Lodash = require("lodash");
var router_1 = require("./router");
var resolver_1 = require("./resolver");
var router_2 = require("./router");
var Fs = require("fs");
var logger_1 = require("./logger");
var validator_1 = require("./validator");
var kernel_1 = require("./kernel");
/**
 * Create instance of KambojaJS application
 */
var Kamboja = /** @class */ (function () {
    /**
     * Create instance of KambojaJS application
     * @param engine KambojaJS engine implementation
     * @param opt KambojaJS option
     */
    function Kamboja(engine, opt) {
        this.engine = engine;
        var override;
        if (typeof opt === "string")
            override = { rootPath: opt };
        else
            override = opt;
        var idResolver = new resolver_1.DefaultIdentifierResolver();
        var pathResolver = new resolver_1.DefaultPathResolver(override.rootPath);
        var resolver = new resolver_1.DefaultDependencyResolver(idResolver, pathResolver);
        var storage = new router_1.MetaDataLoader(idResolver, pathResolver);
        var options = Lodash.assign({
            skipAnalysis: false,
            controllerPaths: ["controller"],
            modelPath: Kamboja.defaultModelPath,
            autoValidation: true,
            rootPath: process.cwd(),
            showLog: "Info",
            validators: [],
            middlewares: [],
            facilities: [],
            identifierResolver: idResolver,
            pathResolver: pathResolver,
            dependencyResolver: resolver,
            metaDataStorage: storage
        }, override);
        this.options = options;
        Kamboja.facade = options;
        this.storage = this.options.metaDataStorage;
    }
    Kamboja.getFacade = function () {
        return Kamboja.facade;
    };
    Kamboja.prototype.set = function (key, value) {
        this.options[key] = value;
        return this;
    };
    Kamboja.prototype.get = function (key) {
        return this.options[key];
    };
    /**
     * Add middleware
     * @param factory factory method that will be call after KambojaJS application initialized
     * @returns KambojaJS application
     */
    Kamboja.prototype.use = function (middleware) {
        this.options.middlewares.push(middleware);
        return this;
    };
    Kamboja.prototype.apply = function (facility) {
        if (typeof facility == "string") {
            try {
                var fac = this.options.dependencyResolver.resolve(facility);
                this.options.facilities.push(fac);
                fac.apply(this);
            }
            catch (e) {
                throw new Error("Unable to instantiate " + facility + " as Facility");
            }
        }
        else {
            this.options.facilities.push(facility);
            facility.apply(this);
        }
        return this;
    };
    Kamboja.prototype.isFolderProvided = function () {
        var _this = this;
        var result = true;
        //controller
        this.options.controllerPaths.forEach(function (x) {
            var path = _this.options.pathResolver.resolve(x);
            if (!Fs.existsSync(path)) {
                result = false;
                _this.log.error("Controller path [" + x + "] provided in configuration is not exist");
            }
        });
        //model
        var modelPath = this.options.pathResolver.resolve(this.options.modelPath);
        if (!Fs.existsSync(modelPath) && this.options.modelPath != Kamboja.defaultModelPath) {
            this.log.error("Model path [" + this.options.modelPath + "] provided in configuration is not exist");
            result = false;
        }
        return result;
    };
    Kamboja.prototype.generateRoutes = function (controllerMeta) {
        var route = new router_2.RouteGenerator(this.options.identifierResolver, controllerMeta);
        var infos = route.getRoutes();
        if (infos.length == 0) {
            var paths = this.options.controllerPaths.join(",");
            this.log.error("No controller found in [" + paths + "]");
        }
        return infos;
    };
    Kamboja.prototype.analyzeRoutes = function (infos) {
        var _this = this;
        var routeAnalyzer = new router_2.RouteAnalyzer(infos);
        var analysis = routeAnalyzer.analyse();
        for (var _i = 0, analysis_1 = analysis; _i < analysis_1.length; _i++) {
            var item = analysis_1[_i];
            if (item.type == "Warning")
                this.log.warning(item.message);
            else
                this.log.error(item.message);
        }
        if (analysis.some(function (x) { return x.type == "Error"; })) {
            return false;
        }
        var validRoutes = infos.filter(function (x) { return !x.analysis; });
        if (validRoutes.length == 0) {
            var path = this.options.controllerPaths.join(", ");
            this.log.newLine().error("No valid controller found in [" + path + "]");
            return false;
        }
        this.log.newLine().info("Routes generated successfully");
        this.log.info("--------------------------------------");
        validRoutes.forEach(function (x) {
            var method = "";
            switch (x.httpMethod) {
                case "GET":
                    method = "GET    ";
                    break;
                case "PUT":
                    method = "PUT    ";
                    break;
                case "PATCH":
                    method = "PATCH  ";
                    break;
                case "POST":
                    method = "POST   ";
                    break;
                case "DELETE":
                    method = "DELETE ";
                    break;
                case "EVENT":
                    method = "EVENT  ";
                    break;
            }
            _this.log.info(method + " " + x.route);
        });
        this.log.info("--------------------------------------");
        this.options.routeInfos = validRoutes;
        return true;
    };
    Kamboja.prototype.resolveMiddlewares = function (option) {
        option.middlewares = kernel_1.MiddlewareFactory.resolveArray(option.middlewares, option.dependencyResolver);
    };
    Kamboja.prototype.resolveValidators = function (option) {
        option.validators = validator_1.ValidatorFactory.resolve(option.validators, option.dependencyResolver);
    };
    /**
     * Initialize KambojaJS application
     * @returns HttpServer
     */
    Kamboja.prototype.init = function (app) {
        this.log = new logger_1.Logger(this.options.showLog);
        if (!this.isFolderProvided())
            throw new Error("Fatal error");
        this.storage.load(this.options.controllerPaths, "Controller");
        this.storage.load(this.options.modelPath, "Model");
        //TODO: possibly temporal coupling
        this.resolveValidators(this.options);
        this.resolveMiddlewares(this.options);
        var routeInfos = this.generateRoutes(this.storage.getFiles("Controller"));
        if (routeInfos.length == 0)
            throw new Error("Fatal error");
        if (!this.analyzeRoutes(routeInfos))
            throw new Error("Fatal Error");
        var httpControllers = this.options.routeInfos.filter(function (x) { return x.httpMethod != "EVENT"; });
        var httpApp = this.engine.init(httpControllers, this.options, app);
        this.set("httpApp", httpApp);
        if (this.options.socketEngine) {
            var socketControllers = routeInfos.filter(function (x) { return x.httpMethod == "EVENT"; });
            var socketApp = this.options.socketEngine.init(socketControllers, this.options, httpApp);
            this.set("socketApp", socketApp);
        }
        return httpApp;
    };
    Kamboja.defaultModelPath = "model";
    return Kamboja;
}());
exports.Kamboja = Kamboja;
